<?php

declare(strict_types=1);

namespace Drupal\field_ipaddress_pgsql\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use IPTools\Network;

/**
 * Plugin implementation of the 'IP Address MySQL' formatter.
 *
 * @FieldFormatter(
 *   id = "ipaddress_mysql",
 *   label = @Translation("Default"),
 *   field_types = {
 *     "ipaddress_mysql"
 *   }
 * )
 */
final class IpAddressMySqlFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    return [
      'function' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $form = parent::settingsForm($form, $form_state);

    $options = [
      'host' => 'host',
      'masklen' => 'masklen',
      'netmask' => 'netmask',
      'network' => 'network',
      'text' => 'text',
    ];

    $form['function'] = [
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => $this->getSetting('function'),
      '#empty_value' => '',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    return [
      $this->t('IP Address function to display additional information about the IP address.'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $element = [];

    foreach ($items as $delta => $item) {
      $ipAddress = inet_ntop($item->ip_address);
      $networkMask = inet_ntop($item->network_mask);
      $element[$delta] = $this->viewValue($ipAddress, $networkMask);
    }

    return $element;
  }

  /**
   * Generate the output appropriate for one field item.
   *
   * @param string $ipAddress
   *   The IP address.
   * @param string $networkMask
   *   The network mask.
   *
   * @return array
   *   The textual output generated as a render array.
   */
  protected function viewValue($ipAddress, $networkMask) {
    // If a user has not selected any functions in the formatter settings, then
    // return the IP address in the CIDR notation, like "192.168.100.1/24".
    if ($this->getSetting('function') === '') {
      $prefixLength = Network::parse("$ipAddress $networkMask")->getPrefixLength();
      return [
        '#theme' => 'field__ipaddress',
        '#value' => "$ipAddress/$prefixLength",
      ];
    }

    $function = $this->getSetting('function');
    $value = '';

    switch ($function) {
      case 'host':
        $value = Network::parse("$ipAddress $networkMask")->getIP();
        break;

      case 'masklen':
        $cidr = Network::parse("$ipAddress $networkMask")->getCIDR();
        $parts = explode('/', $cidr);
        $value = $parts[1];
        break;

      case 'netmask':
        $value = $networkMask;
        break;

      case 'network':
        $value = Network::parse("$ipAddress $networkMask")->getNetwork();
        break;

      case 'text':
        $value = $ipAddress . ' ' . $networkMask;
    }

    return [
      '#theme' => 'field__ipaddress',
      '#value' => $value,
    ];

  }

}
