<?php

declare(strict_types=1);

namespace Drupal\field_ipaddress_pgsql\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Plugin implementation of the 'IP Address MySQL' field type.
 *
 * @FieldType(
 *   id = "ipaddress_mysql",
 *   label = @Translation("IP address"),
 *   description = @Translation("Stores an IP address."),
 *   weight = -100,
 *   default_widget = "ipaddress_mysql",
 *   default_formatter = "ipaddress_mysql",
 *   constraints = {"MySqlFormat" = {}}
 * )
 */
final class IpAddressMySqlField extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings(): array {
    return [
      'family' => '4',
      'cidr_format' => FALSE,
    ] + parent::defaultStorageSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function storageSettingsForm(array &$form, FormStateInterface $form_state, $has_data) {
    $element = [];

    $element['family'] = [
      '#type' => 'radios',
      '#title' => $this->t('IP version allowed'),
      '#options' => [
        '4' => $this->t('IPv4'),
        '6' => $this->t('IPv6'),
      ],
      '#description' => $this->t('Select the IP address family that are allowed.'),
      '#default_value' => $this->getSetting('family'),
      '#disabled' => $has_data,
    ];

    $description = $this->t('If checked, only allow addresses that comply with <a href=":url" target="_blank">CIDR</a> notation.', [
      ':url' => 'https://www.postgresql.org/docs/16/datatype-net-types.html#DATATYPE-CIDR',
    ]);

    $element['cidr_format'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('CIDR'),
      '#description' => $description,
      '#default_value' => $this->getSetting('cidr_format'),
      '#disabled' => $has_data,
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty(): bool {
    return match ($this->get('ip_address')->getValue()) {
      NULL, '' => TRUE,
      default => FALSE,
    };
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition): array {
    $properties = [];
    $properties['ip_address'] = DataDefinition::create('any')
      ->setLabel(t('IP address'))
      ->setDescription(t('The IP address.'));

    $properties['network_mask'] = DataDefinition::create('any')
      ->setLabel(t('Network mask'))
      ->setDescription(t('The network mask.'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition): array {
    // We add the "type" key just to prevent error messages like "Warning:
    // Undefined array key "type" in views_field_default_views_data()..."
    $columns = [
      'ip_address' => [
        'mysql_type' => 'VARBINARY(16)',
        'not null' => TRUE,
        'description' => 'The IP address.',
        'type' => '',
      ],
      'network_mask' => [
        'mysql_type' => 'VARBINARY(16)',
        'not null' => TRUE,
        'description' => 'The network mask.',
        'type' => '',
      ],
    ];

    $indexes = [
      'ip_address' => ['ip_address'],
      'network_mask' => ['network_mask'],
    ];

    $schema = [
      'columns' => $columns,
      'indexes' => $indexes,
    ];

    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition): array {
    $values['value'] = '192.168.0.1/24';
    return $values;
  }

}
